/**
 * @author Sergey Chikuyonok (serge.che@gmail.com)
 * @link http://chikuyonok.ru
 * @description
 * <b>Как это работает</b>
 * <code>
 * var anim = spriteAnimation(elem, frame_size, reel_size, props);
 * anim.start();
 * </code>
 * <br><br>
 * 
 * <b>Параметры</b>
 * <code>elem</code> : <i>Element</i> — указатель на HTML-элемент, который 
 * нужно анимировать<br>
 * 
 * <code>frame_size</code> : <i>Number, Object</i> — размер кадра, в пикселях. 
 * Так как анимация может работать по оси X и Y, контроллеру нужно знать ширину 
 * и высоту кадра. Можно передавать число (будет являться шириной и высотой), 
 * либо объект со свойствами <code>width</code> и <code>height</code>.<br>
 * 
 * <code>reel_size</code> : <i>Number, Object</i> — Размер всего спрайта 
 * (картинки), в пикселях. Аналогично параметру <code>frame_size</code> можно 
 * передавать число или объект.<br>
 * 
 * <code>props</code> : <i>String, Array</i> — CSS-свойства элемента elem, 
 * которые нужно анимировать (принимаются ‘left’, ‘top’, ‘backgroundPositionX’, 
 * ‘backgroundPositionY’). Можно передавать массив строк, а можно передать одну 
 * строку, где все свойства разделены пробелом (например ‘top backgroundPositionX’)<br>
 * 
 * <b>Доступные методы и свойства возвращаемого объекта:</b>
 * <code>start()</code> : <i>void</i> — Запустить анимацию.<br>
 * <code>stop()</code> : <i>void</i> — Остановить анимацию.<br>
 * <code>isAnimating()</code> : <i>Boolean</i> — Проверяет, работает ли анимация в данный момент
 * <code>frameDelay(value)</code> : <i>Number</i> — Устанавливает (если передан 
 * аргумент value) и возвращает задержку между сменой кадров, в милисекундах. 
 * Можно задавать отрицательные значения: это означает, что спрайт должен 
 * двигаться в обратную сторону.<br>
 * <code>elem</code> : <i>Element</i> — Анимируемый объект<br>
 * <code>getProperties()</code> : <i>Array</i> — Список анимируемых CSS-свойств
 * <code>frameSize(value)</code> : <i>Object</i> — Устанавливает (если передан 
 * аргумент value) и возвращает размер кадра (объект со свойствами 
 * <code>width</code> и <code>height</code>).<br><br>
 * 
 * @example
 * <code>
 * spriteAnimation(document.getElementById('sample'), 100, 1600, 'left').start();
 * 
 * var anim = spriteAnimation(document.getElementById('sample2'), {width: 100, height: 200}, 1600, 'top backgroundPositionX');
 * anim.frameDelay(80);
 * anim.start();
 * 
 * button.onclick = function(){
 * 	if (anim.isAnimating())
 * 		anim.stop();
 * }
 * </code>
 */
(function(){
	/** Массив элементов, которые нужно анимировать */
	var anim_items = [],
		
		/** Базовая задержка перед сменой кадров, мс */
		default_frame_delay = 33,
		
		bg_x = 'backgroundPositionX', 
		bg_y = 'backgroundPositionY',
		
		prop_types = {
			x: ['left', bg_x],
			y: ['top', bg_y]
		},
		
		id_counter = 0;
	
	var userAgent = navigator.userAgent.toLowerCase();

	/** @type {Boolean} Говорит, что текущий браузер — Internet Explorer */
	var is_msie = /msie/.test(userAgent) && !/opera/.test(userAgent);
			
	/**
	 * Возвращает значение CSS-свойства <code>name</code> элемента <code>elem</code>
	 * @author John Resig (http://ejohn.org)
	 * @param {Element} elem Элемент, у которого нужно получить значение CSS-свойства
	 * @param {String} name Название CSS-свойства
	 * @return {String}
	 */
	function getStyle(elem, name) {
		// If the property exists in style[], then it's been set
		// recently (and is current)
		if (elem.style[name]) {
			return elem.style[name];
		}
		
		//Otherwise, try to use IE's method
		else if (is_msie) {
			return elem.currentStyle[name];
		}
		// Or the W3C's method, if it exists
		else if (document.defaultView && document.defaultView.getComputedStyle) {
			//It uses the traditional 'text-align' style of rule writing,
			//instead of textAlign
			name = name.replace(/([A-Z])/g, "-$1").toLowerCase();
			// Get the style object and get the value of the property (if it exists)
			var s = document.defaultView.getComputedStyle(elem, "");
			return s && s.getPropertyValue(name);
		}
		//Otherwise, we're using some other browser
		else {
			return null;
		}
	}
	
	/**
	 * Проверяет, содержит ли массив <code>array</code> значение <code>value</code>
	 * @param {Array} array
	 * @param {Object} value
	 */
	function hasValue(array, value) {
		for (var i = 0; i < array.length; i++) 
			if (array[i] === value)
				return true;
		
		return false;
	}
	
	/**
	 * Парсит анимируемые свойства и приводит их к общей нотации. Если передана
	 * строка, то она разбивается на фрагменты по пробелу или запятой
	 * @param {String|Array} props Анимируемые CSS-свойства
	 * @return {Array}
	 */
	function parseProperties(props) {
		var result = [];
		if (typeof(props) == 'string') 
			props = props.split(/\s|,/);
		
		for (var i = 0; i < props.length; i++) {
			if (props[i])
				result.push(props[i].replace(/\-(\w)/g, function(str, p1){ return p1.toUpperCase(); }));
		}
		
		return result;
	}
	
	/**
	 * Возвращает текущее время в милисекундах
	 * @return {Number}
	 */
	function getTime() {
		return (new Date).getTime();
	}
	
	/**
	 * Парсит переданное значение, получая из него значения ширины и высоты
	 * @param {Number|Object} value 
	 */
	function parseSize(value) {
		var result = {width: 0, height: 0};
		if (typeof(value) == 'number' || typeof(value) == 'string')
			result.width = result.height = parseInt(value, 10);
		else {
			result.width = value.width || 0;
			result.height = value.height || 0;
		}
		
		result.x = result.width;
		result.y = result.height;
		
		return result;
	}
	
	/**
	 * Основной цикл работы контроллера
	 */
	function controllerLoop() {
		if (anim_items.length) {
			var time = getTime();
			for (var i = 0; i < anim_items.length; i++) {
				if (anim_items[i].time <= time) {
					runSprite(anim_items[i].sprite, anim_items[i]);
					anim_items[i].time = time + Math.abs(anim_items[i].sprite.frameDelay());
				}
			}
			
			setTimeout(controllerLoop, 5);
		}
	}
	
	/**
	 * Смещает спрайт в следующий кадр
	 * @param {spriteAnimation()} sprite Анимационный объект спрайта
	 * @param {Object} anim_obj Вспомогательный элемент, содержащий информацию о текущем состоянии анимации
	 */
	function runSprite(sprite, anim_obj) {
		/** Направление анимации */
		var offset_dir = (sprite.frameDelay() < 0) ? 'right' : 'left',
			props = sprite.getProperties(),
			bg_pos,
			elem = sprite.elem;
		
		// из-за особенностей работы со свойствами backgroud-position-* необходимо
		// заранее получить их свойства
		if (hasBgValue(props)) 
			bg_pos = getBgPos(sprite.elem);
		
		// меняем значения кадров
		for (var i = 0; i < props.length; i++) {
			var k = props[i],
				axis = getPropertyAxis(k),
				cur_frame = anim_obj.cur_frame[k] + ( offset_dir == 'left' ? 1 : -1 );
			if (cur_frame >= anim_obj.frame_count[axis]) {
				if (anim_obj.looped)
					cur_frame = 0;
				else
					cur_frame = anim_obj.frame_count[axis] - 1;
			} else if (cur_frame < 0) {
				if (anim_obj.looped)
					cur_frame = anim_obj.frame_count[axis] - 1;
				else
					cur_frame = 0;
			}
			anim_obj.cur_frame[k] = cur_frame;
		}
		
		// выставляем значения свойств
		for (var i = 0; i < props.length; i++) {
			var k = props[i];
				
			if (k != bg_x && k != bg_y) 
				elem.style[k] = (-sprite.frameSize()[getPropertyAxis(k)] * anim_obj.cur_frame[k]) + 'px';
		}
		
		if (bg_pos) {
			var bg_pos_x = (bg_x in anim_obj.cur_frame) ? -sprite.frameSize().x * anim_obj.cur_frame[bg_x] : bg_pos[bg_x];
			var bg_pos_y = (bg_y in anim_obj.cur_frame) ? -sprite.frameSize().y * anim_obj.cur_frame[bg_y] : bg_pos[bg_y];
			elem.style.backgroundPosition = bg_pos_x + 'px ' + bg_pos_y + 'px';
		}
	}
	
	/**
	 * Возвращает координаты фона
	 * @param {Element} elem 
	 */
	function getBgPos(elem) {
		var bg_pos = getStyle(elem, 'backgroundPosition'),
			bg = {backgroundPositionX: 0, backgroundPositionY: 0};
		
		if (bg_pos) {
			bg_pos = bg_pos.split(' ', 2);
			bg[bg_x] = parseInt(bg_pos[0], 10);
			bg[bg_y] = parseInt(bg_pos[1], 10);
		}
		
		return bg;
	}
	
	/**
	 * Добавляет объект в список анимируемых объектов
	 * @param {Object} obj Анимируемый объект
	 */
	function addAnimation(obj) {
		obj.time = 0;
		anim_items.push(obj);
		
		if (anim_items.length == 1)
			controllerLoop();
	}
	
	/**
	 * Удаляет объект из списка анимируемых объектов
	 * @param {Number} id ID анимируемого объекта
	 */
	function removeAnimation(id) {
		for (var i = 0; i < anim_items.length; i++) {
			if (anim_items[i].id == id) {
				anim_items.splice(i, 1);
				break;
			}
		}
	}
	
	/**
	 * Проверяет, содержаттся ли background-position-* свойства среди анимируемых
	 * CSS-свойств
	 * @param {Array} props Анимируемы CSS-свойства
	 * @return {Boolean}
	 */
	function hasBgValue(props) {
		return hasValue(props, bg_x) || hasValue(props, bg_y);
	}
	
	/**
	 * Возвращает ось CSS-свойства ('x' или 'y')
	 * @param {String} prop CSS-свойство
	 * @return {String}
	 */
	function getPropertyAxis(prop) {
		return hasValue(prop_types.y, prop) ? 'y' : 'x';
	}
	
	/**
	 * Высчитывает у элемента текущий кадр для каждого анимируемого CSS-свойства.
	 * Используется при инициализации анимационного объекта чтобы знать, откуда 
	 * начинать анимацию
	 * @param {Element} elem Анимируемый элемент
	 * @param {Object} frame_size Размер кадра
	 * @param {Array} props Анимируемые CSS-свойства
	 * @return {Object} Хэш, где ключом является анимируемое свойство, а значением — номер кадра
	 */
	function calculateCurrentFrame(elem, frame_size, props) {
		var result = {}, bg_pos;
		if (hasBgValue(props))
			bg_pos = getBgPos(elem);
			
		for (var i = 0; i < props.length; i++) {
			var k = props[i];
			var value = (k.indexOf('background') != -1) ? bg_pos[k] : (getStyle(elem, k) || 0);
			value = (value == 'auto') ? 0 : parseInt(value, 10);
			result[k] = Math.round(value / frame_size[getPropertyAxis(k)]);
		}
		
		return result;
	}
	
	/**
	 * Контроллер для анимации спрайта
	 * @param {Element} elem Элемент, который нужно анимировать
	 * @param {Number|Object} frame_size Видимый размер кадра
	 * @param {Number|Object} reel_size Размер спрайта
	 * @param {String|Array} props Набор анимируемых CSS-свойств
	 */
	function spriteAnimation(elem, frame_size, reel_size, props, _looped) {
		frame_size = parseSize(frame_size);
		reel_size = parseSize(reel_size);
		props = parseProperties(props);
		if (_looped === null)
			_looped = true;
		
		/** Флаг, указывающий, работает ли сейчас анимация */
		var is_animating = false,
			
			/** Задержка перед переходами между кадрами */
			delay = default_frame_delay,
			
			id = ++id_counter;
		
		var worker = {
			start: function() {
				if (!is_animating) {
					addAnimation(anim_obj);
					is_animating = true;
				}
			},
			
			stop: function() {
				if (is_animating) {
					removeAnimation(id);
					is_animating = false;
				}
			},
			
			/**
			 * Проверяет, работает ли анимация в данный момент
			 * @return {Boolean}
			 */
			isAnimating: function() {
				return is_animating;
			},
			
			/**
			 * Устанавливает или возвращает задержку между кадрами
			 * @param {Number} value Задержка между кадрами, мс
			 * @return {Number}
			 */
			frameDelay: function(value) {
				if (arguments.length)
					delay = parseInt(value, 10);
				return delay;
			},
			
			/** Анимируемый элемент */
			elem: elem,
			
			/**
			 * Возвращает анимируемые свойства
			 * @return {String[]}
			 */
			getProperties: function() {
				return props;
			},
			
			/**
			 * Возвращает или устанавливает размер кадра
			 * @param {Number|Object} value
			 * @return {Object} Объект с координатами <code>width</code> и <code>height</code>
			 */
			frameSize: function(value) {
				if (arguments.length) {
					frame_size = parseSize(value);
					anim_obj.frame_count.x = (reel_size.width / frame_size.width);
					anim_obj.frame_count.y = (reel_size.height / frame_size.height);
				}
				
				return frame_size;
			}
		};
		
		/**
		 * Объект, содержащий информацию об анимации
		 */
		var anim_obj = {
			/**
			 * Количество кадров по обоим измерениям
			 */
			frame_count: {x: 0,	y: 0},
			
			/**
			 * Текущие кадры для всех свойств 
			 */
			cur_frame: calculateCurrentFrame(elem, frame_size, props),
			
			sprite: worker,
			
			/** Время, когда нужно поменять кадр */
			time: 0,
			
			id: id,
			
			looped: _looped
		}
		
		worker.frameSize(frame_size);
		
		return worker;
	}
	
	window.spriteAnimation = spriteAnimation;
})();
